<?php
// $Id: luceneapi.drush.inc,v 1.4 2009/02/12 05:04:37 cpliakas Exp $

/**
 * @file
 * Drush integration for modules that implement the Search Lucene API.
 */

/**
 * Returns array of drush commands.
 *
 * @return
 *   An array of drush command definitions.
 */
function _luceneapi_drush_get_commands() {
  $items = array();

  // command to update Lucene index
  $items['luceneapi update'] = array(
    'callback' => '_luceneapi_drush_update_index',
    'description' => t('Update a Lucene index'),
  );

  // command to optimize Lucene index
  $items['luceneapi optimize'] = array(
    'callback' => '_luceneapi_drush_optimize',
    'description' => t('Optimizes a Lucene index'),
  );

  // command to return the size of an index
  $items['luceneapi size'] = array(
    'callback' => '_luceneapi_drush_index_size',
    'description' => t('Returns the number of documents in a Lucene index'),
  );

  // command to return the size of an index
  $items['luceneapi wipe'] = array(
    'callback' => '_luceneapi_drush_index_wipe',
    'description' => t('Wipes a lucene index'),
  );

  // command to clear the search results cache
  $items['luceneapi cache clear'] = array(
    'callback' => '_luceneapi_drush_cache_clear',
    'description' => t('Clears the search results cache')
  );

  return $items;
}

/**
 * @defgroup _luceneapi_drush Drush command callbacks.
 * @{
 */

/**
 * Updates Lucene indexes.
 *
 * @param ...
 *   Strings containing modules defining Lucene indexes.
 * @return
 *   NULL
 */
function _luceneapi_drush_update_index() {
  $args = func_get_args();
  if (!count($args)) {
    _luceneapi_drush_usage_error('update');
    return;
  }

  // invokes hook_update_index() on all Search Lucene API modules
  $valid = luceneapi_module_list();
  $modules = ('all' == $args[0]) ? $valid : $args;
  foreach ($modules as $module) {
    if (in_array($module, $valid)) {
      if (module_hook($module, 'update_index')) {
        luceneapi_reset_error();
        module_invoke($module, 'update_index');
        if (!luceneapi_is_error()) {
          drush_print(t(
            'Index successfully updated for !module module',
            array('!module' => $module)
          ));
        }
      }
      else {
        $errstr = sprintf('%s does not implement hook_update_index()', $module);
        luceneapi_throw_error($errstr, 2);
      }
    }
    else {
      luceneapi_throw_error(_luceneapi_get_hook_error($module), 2);
    }
  }
}

/**
 * Optimizes Lucene indexes.
 *
 * This module calls $index->commit() after the index is optimized to save
 * memory.  The index normally performs tasks at shutdown, so this forces the
 * tasks to complete before the next index is opened.
 *
 * @param ...
 *   Strings containing modules defining Lucene indexes.
 * @return
 *   NULL
 */
function _luceneapi_drush_optimize() {
  $args = func_get_args();
  if (!count($args)) {
    _luceneapi_drush_usage_error('optimize');
    return;
  }

  // gets unique index paths from passed modules, updates indexes
  foreach (_luceneapi_drush_get_paths($args) as $path) {
    try {
      if ($index = luceneapi_open_index($path, $errstr, $errno)) {
        luceneapi_optimize_index($index, TRUE);
        $index->commit();

        // formats "success" message
        $modules = luceneapi_path_module_list($path);
        $base_msg = sprintf(
          'Index successfully optimized for %s',
          implode(', ', $modules)
        );
        drush_print(format_plural(
          count($modules), "$base_msg module", "$base_msg modules"
        ));
      }
      else {
        throw new LuceneAPI_Exception($errstr, $errno);
      }
    }
    catch (LuceneAPI_Exception $e) {
      luceneapi_throw_error($e);
    }
  }
}

/**
 * Returns size of a Lucene index.
 *
 * @param ...
 *   Strings containing modules defining Lucene indexes.
 * @return
 *   NULL
 */
function _luceneapi_drush_index_size() {
  $args = func_get_args();
  if (!count($args)) {
    _luceneapi_drush_usage_error('size');
    return;
  }
  if (FALSE !== ($size = luceneapi_index_size($args[0]))) {
    drush_print($size);
  }
}

/**
 * Wipes an index.
 *
 * @param ...
 *   Strings containing modules defining Lucene indexes.
 * @return
 *   NULL
 */
function _luceneapi_drush_index_wipe() {
  $args = func_get_args();
  if (!count($args)) {
    _luceneapi_drush_usage_error('wipe');
    return;
  }

  // makes sure the user wants to wipe out the index
  if (!drush_confirm(t('Are you sure you want to wipe out the index?'))) {
    drush_print(t('Operation aborted'));
    return;
  }

  // invokes hook_search('reset') on all Search Lucene API modules
  $valid = luceneapi_module_list();
  $modules = ('all' == $args[0]) ? $valid : $args;
  foreach ($modules as $module) {
    if (in_array($module, $valid)) {
      $hook_search = sprintf('%s_search', $module);
      if (function_exists($hook_search)) {

        // resets error flag, calls hook search
        luceneapi_reset_error();
        $hook_search('reset');

        // formats "success" message if no errors
        if (!luceneapi_is_error()) {
          drush_print(sprintf('Index successfully wiped for %s', $module));
        }
      }
      else {
        $errstr = sprintf('%s does not implement hook_search()', $module);
        luceneapi_throw_error($errstr, 2);
      }
    }
    else {
      luceneapi_throw_error(_luceneapi_get_hook_error($module), 2);
    }
  }
}

/**
 * Clears the search results cache.
 *
 * @param ...
 *   Strings containing modules defining Lucene indexes.
 * @return
 *   NULL
 */
function _luceneapi_drush_cache_clear() {
  $args = func_get_args();
  if (!count($args)) {
    _luceneapi_drush_usage_error('cache clear');
    return;
  }

  // clears a module's cached results in the search lucene api cache table
  $valid = luceneapi_module_list();
  $modules = ('all' == $args[0]) ? $valid : $args;
  foreach ($modules as $module) {
    if (in_array($module, $valid)) {
      cache_clear_all($module, LUCENEAPI_CACHE_TABLE, TRUE);
      drush_print(sprintf(
        'Search results cache successfully cleared for %s', $module
      ));
    }
    else {
      luceneapi_throw_error(_luceneapi_get_hook_error($module), 2);
    }
  }
}

/**
 * Returns array of unique index paths from passed modules.  This function
 * prevents indexes from being indexed twice if mutiple modules that use the
 * same index are passed.
 *
 * @param $args
 *   An array of arguments passed by the drush command.
 * @return
 *   An array of unique index paths.
 */
function _luceneapi_drush_get_paths(array $args) {
  if ('all' == $args[0]) {
    return luceneapi_path_list();
  }
  else {
    $return = array();
    $paths = luceneapi_module_list(TRUE);
    foreach ($args as $module) {
      if (array_key_exists($module, $paths)) {
        if (!in_array($paths[$module], $return)) {
          $return[] = $paths[$module];
        }
      }
      else {
        luceneapi_throw_error(_luceneapi_get_hook_error($module), 2);
      }
    }
  }
  return $return;
}

/**
 * Throws a drush usage error.
 *
 * @param $subcommand
 *   A string containing the luceneapi subcommand.
 * @return
 *   NULL
 */
function _luceneapi_drush_usage_error($subcommand) {
  $section = sprintf('drush:luceneapi %s', $subcommand);
  drush_error('Usage: '. luceneapi_help($section));
}

/**
 * @} End of "defgroup _luceneapi_drush".
 */
